# ***************************************************************************
# Copyright 2019 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import json
import nsx_logging
from cli.utils.command_utils import invoke_command
from cli.utils.node_utils import get_node_type
from cli.utils.constants import NODE_TYPE_HOST_CENTOSKVM
from cli.utils.constants import NODE_TYPE_HOST_SLESKVM
from cli.utils.constants import NODE_TYPE_HOST_RHELKVM
from cli.utils.constants import NODE_TYPE_HOST_UBUNTUKVM
from cli.utils.constants import NODE_TYPE_HOST_WIN
from cli.utils.constants import NODE_TYPE_BAREMETAL_UBUNTU_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_SLES_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_CONTAINER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_CONTAINER
from cli.utils.location_utils import FilePaths
from cli.utils.error_code import ErrorCode as ec
from cli.commands.cli_command_provider import CliCommandProvider
from cli.commands.host_shared.firewall import DFWVifArgument

lg = nsx_logging.getLogger("cli.commands.host_kvm.firewallthresholds")

#
# DFW threshold related commands
#

_FIREWALL_THRESHOLDS_CLI_KVM = {
    NODE_TYPE_HOST_SLESKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_HOST_CENTOSKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_HOST_RHELKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_HOST_UBUNTUKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_HOST_WIN: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_UBUNTU_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_RHEL_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_CENTOS_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_SLES_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_RHEL_CONTAINER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_CENTOS_CONTAINER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    }

def has_firewall_thresholds_cli_kvm(node_type):
   return (node_type == NODE_TYPE_HOST_SLESKVM or
           node_type == NODE_TYPE_HOST_CENTOSKVM or
           node_type == NODE_TYPE_HOST_RHELKVM or
           node_type == NODE_TYPE_HOST_UBUNTUKVM or
           node_type == NODE_TYPE_BAREMETAL_UBUNTU_SERVER or
           node_type == NODE_TYPE_BAREMETAL_RHEL_SERVER or
           node_type == NODE_TYPE_BAREMETAL_CENTOS_SERVER or
           node_type == NODE_TYPE_BAREMETAL_SLES_SERVER or
           node_type == NODE_TYPE_BAREMETAL_RHEL_CONTAINER or
           node_type == NODE_TYPE_BAREMETAL_CENTOS_CONTAINER or
           node_type == NODE_TYPE_HOST_WIN)


def invoke_firewall_thresholds_command_kvm(cmd_list):
    node_type = get_node_type()
    if has_firewall_thresholds_cli_kvm(node_type):
        cmd = [_FIREWALL_THRESHOLDS_CLI_KVM[node_type]['bin'], "-t", _FIREWALL_THRESHOLDS_CLI_KVM[node_type]['socket']]
        cmd.extend(cmd_list)
        rc, out, err = invoke_command(cmd)
        return (rc, out, err)
    else:
        lg.error("node type %s does not have firewall thresholds CLI", node_type,
                 error_code=ec.INVALID_ARGUMENT)
        return (-1, None, "invalid command")


def get_firewall_thresholds_cli_output_kvm(command):
    cmd_list = [command]
    rc, out, err = invoke_firewall_thresholds_command_kvm(cmd_list)
    if rc != 0:
        lg.error("Failed to invoke firewall thresholds KVM CLI command, rc: %d, err: %s", rc, err,
                 error_code=ec.OPERATION_FAILED)
        out = None
    return out

# commands

class FirewallShowThresholdsKVM(CliCommandProvider):
    CMD_STR = "get/thresholds"
    def __call__(self, cmd_list):
        lg.debug("%s: cmd_list cnt=%d" % (self.__class__.__name__, len(cmd_list)))
        if self.CMD_STR:
            res = get_firewall_thresholds_cli_output_kvm(self.CMD_STR)
            cmd_result = ""
            if res is None:
                cmd_result = "MSG_FIREWALLTHRESHOLD_COMM_ERROR"
            elif "not available" in res:
                cmd_result = "MSG_FIREWALLTHRESHOLD_NOT_AVAILABLE"
            else:
                cmd_result = "OK\n\n%s" % res
        else:
            cmd_result = "MSG_FIREWALLTHRESHOLD_COMM_ERROR"
        return cmd_result

