import nsx_logging
from cli.utils.command_utils import invoke_command
from cli.utils.constants import NODE_TYPE_HOST_ESX
from cli.utils.node_utils import get_node_type
from cli.utils.location_utils import FilePaths
from cli.utils.error_code import ErrorCode as ec

lg = nsx_logging.getLogger("cli.commands.lcp_common.cfgagent.client")

MAXOUTPUTSIZE = 8388608  # 8 MB

_CFGAGENT_CLI = {
    NODE_TYPE_HOST_ESX: {
        'bin': "/opt/vmware/nsx-cli/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-cfgagent/cfgagent-cli"
        },
    }


def has_cfgagent_cli(node_type):
    return (node_type == NODE_TYPE_HOST_ESX)


def invoke_cfgagent_appctl_command(cmd_list):
    node_type = get_node_type()
    if has_cfgagent_cli(node_type):
        cmd = [_CFGAGENT_CLI[node_type]['bin'], "-t", _CFGAGENT_CLI[node_type]['socket']]
        cmd.extend(cmd_list)
        rc, out, err = invoke_command(cmd, max_out_size=MAXOUTPUTSIZE)
        return (rc, out, err)
    else:
        lg.error("Node type: %s does not support running"
                 "the cfgagent CLI command: %s" , node_type, cmd_list, error_code=ec.OPERATION_NOT_SUPPORTED)
        return (-1, None, "APPCTL_CLI_INVALID_COMMAND")


def get_cfgagent_cli_output(command, argv=[]):
    cmd_list = [command]
    if (len(argv)) > 0:
        cmd_list.extend(argv)
    rc, out, err = invoke_cfgagent_appctl_command(cmd_list)
    if rc != 0:
        lg.error("Failed to invoke cfgagent CLI command, rc: %d, err: %s", rc, err,
                 error_code=ec.OPERATION_FAILED)
        out = None

    return out
