# Copyright (C) 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

import json
import sys
import traceback

from sha.contrib.metric.edge_base import EdgeBase
from sha.contrib.metric.utils import run_command, timeout_cache

class IdpsStats(EdgeBase):

    def get_idps_stats(self):
        # Get edge IDPS stats
        cmd_get_stats = ['nsx-appctl', '-t', '/var/run/vmware/nsx-opsagent/opsagent-cli', 'get/edgeidsStats']
        try:
            output = self.run_command_with_time_check(cmd_get_stats, timeout=10)
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.warning("Failed to get edge IDPS stats: {}, {}".format(e, trace))
            output = None
        if output is None:
            edge_idps_stats = None
        else:
            edge_idps_stats = json.loads(output)
        self._logger.debug("Edge IDPS stats({}): {}".format(sys.getsizeof(edge_idps_stats), edge_idps_stats))

        # Get event plugin stats
        cmd_get_stats = ['edge-appctl', '-t', '/var/run/vmware/edge/dpd.ctl', 'evtpluginstats/get']
        try:
            output = self.run_command_with_time_check(cmd_get_stats, timeout=10)
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.warning("Failed to get IDPS event plugin stats: {}, {}".format(e, trace))
            output = None
        if output is None:
            event_plugin_stats = None
        else:
            event_plugin_stats = json.loads(output)
        self._logger.debug("Edge IDPS Event plugin stats({}): {}".format(sys.getsizeof(event_plugin_stats), event_plugin_stats))

        # Get IDS rule stats
        cmd_get_stats = ['edge-appctl', '-t', '/var/run/vmware/edge/dpd.ctl', 'fw/getidsrulestat']
        try:
            output = self.run_command_with_time_check(cmd_get_stats, timeout=10)
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.warning("Failed to get IDPS rule stats: {}, {}".format(e, trace))
            output = None
        if output is None:
            idps_rule_stats = None
        else:
            idps_rule_stats = json.loads(output)
        self._logger.debug("Edge IDS rule stats({}): {}".format(sys.getsizeof(idps_rule_stats), idps_rule_stats))

        return {'edge-idps-stats': edge_idps_stats, \
                'evt-plugin-stats': event_plugin_stats, \
                'idps-rule-stats': idps_rule_stats}

    @timeout_cache(10)
    def usage(self, *args, **kwargs):
        idps_stats_dict = {}
        try:
            idps_stats_dict = self.get_idps_stats()
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.warning("Failed to get IDPS stats on {}: {}, {}".format(e, trace))
            idps_stats_dict = None

        return idps_stats_dict
