# Copyright (C) 2019 VMware, Inc. All rights reserved.
# -- VMware Confidential
import time
from functools import wraps


def timeout_cache(live_duration):
    def decorator(fn):
        record = [None, None]  # [0] is used to store timestamp, [1] is used to store cache

        # noinspection PyTypeChecker
        @wraps(fn)
        def wrapper(*args, **kwargs):
            if not record[0] or time.monotonic() - record[0] > live_duration:
                record[1] = fn(*args, **kwargs)
                record[0] = time.monotonic()
            return record[1]

        return wrapper

    return decorator
