# Copyright (C) 2019 VMware, Inc. All rights reserved.
# -- VMware Confidential
import sha.config


class CustomPluginCache:
    """
    Cache installed custom plugin for later use. E.g. Resolver will get default profile from cache
    """
    _logger = sha.config.get_logger(__name__)
    _plugins = dict()

    @classmethod
    def add_plugin(cls, plugin):
        if plugin.id in cls._plugins:
            cls._logger.info('Add plugin failed. Plugin with same id already existed :{}'.format(plugin))
            return
        cls._plugins[plugin.id] = plugin
        cls._logger.info('Added plugin: {}, all plugin IDs in cache:{}'.format(plugin, cls._plugins.keys()))

    @classmethod
    def remove_plugin(cls, id_):
        if id_ not in cls._plugins:
            cls._logger.info('Not exist plugin id: {}, no need perform removal.'.format(id_))
            return

        cls._plugins.pop(id_, None)
        cls._logger.info('Removed plugin id: {}, all plugin IDs in cache:{}'.format(id_, cls._plugins.keys()))

    @classmethod
    def get_plugin(cls, id_):
        return cls._plugins.get(id_, None)

    @classmethod
    def plugin_num(cls):
        return len(cls._plugins)
